from ...dbsignal.Signal import Signal
from ...metadata.SignalMetadata import SignalMetadata


class PowerConverterQuery(object):

    @staticmethod
    def query_preprocess_fgc_current(metadata_fgc, SIGNAL_NAME, timestamp):
        i_meas_df = Signal().read('pm', system=metadata_fgc['system'], className=metadata_fgc['className'],
                                  source=metadata_fgc['source'],
                                  signal=SIGNAL_NAME, eventTime=timestamp)
        # Preprocess signal
        return i_meas_df.set_index(((i_meas_df.index.values) - timestamp) / 1e+9)

    @staticmethod
    def get_st_abort_pic_timestamp(circuit_type, circuit_name, timestamp, ldb, duration=[(1, 's'), (60, 's')]):
        ST_ABORT_PIC = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'PIC', 'CALS', 'ST_ABORT_PIC')[0]
        st_abort_pic_df = Signal().read('CALS', signal=ST_ABORT_PIC, t_start=timestamp, duration=duration, ldb=ldb)

        if len(st_abort_pic_df) == 0:
            ST_ABORT_PIC = SignalMetadata.get_signal_name(circuit_type, circuit_name, 'PIC', 'CALS', 'ST_ABORT_PIC')[1]
            st_abort_pic_df = Signal().read('CALS', signal=ST_ABORT_PIC, t_start=timestamp, duration=duration, ldb=ldb)

        if len(st_abort_pic_df) == 0:
            print('Empty signal ST_ABORT_PIC, returning 0!')
            return 0

        return st_abort_pic_df.index[0] * 1e+9

    @staticmethod
    def query_earth_current(metadata_fgc, timestamp_fgc_act, timestamp_fgc_ref):
        I_A = "I_A"
        i_a_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, I_A, timestamp_fgc_act)

        IEARTH = "IEARTH"
        i_earth_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, IEARTH, timestamp_fgc_act)
        i_earth_ref_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, IEARTH, timestamp_fgc_ref)

        return i_a_df, i_earth_df, i_earth_ref_df

    @staticmethod
    def query_earth_current_pcnt(metadata_fgc, timestamp_fgc_act, timestamp_fgc_ref, scaling=1):
        I_EARTH_PCNT = "I_EARTH_PCNT"
        i_earth_pcnt_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, I_EARTH_PCNT,
                                                                           timestamp_fgc_act)
        i_earth_pcnt_ref_df = PowerConverterQuery.query_preprocess_fgc_current(metadata_fgc, I_EARTH_PCNT,
                                                                               timestamp_fgc_ref)
        # scale with respect to PNO.b2
        i_earth_pcnt_ref_df = i_earth_pcnt_ref_df * scaling

        return i_earth_pcnt_df, i_earth_pcnt_ref_df
